/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2012 digitalSTROM.org, Zurich, Switzerland
 *  Author: Roman Köhler
 */

function getParameterByName(name)
{
  name = name.replace(/[\[]/, "\\\[").replace(/[\]]/, "\\\]");
  var regexS = "[\\?&]" + name + "=([^&#]*)";
  var regex = new RegExp(regexS);
  var results = regex.exec(window.location.search);
  if(results == null)
    return "";
  else
    return decodeURIComponent(results[1].replace(/\+/g, " "));
}

function getStateGridTextActive (scene)
{
    var scene_name = dss.staticDataModel.getNameForScene(0, 0, scene);
    var gridTextActive = Ext.String.format2(_("only when {name} is active"), {'name': scene_name});
    return gridTextActive;
}

function getStateGridTextInactive (scene)
{
    var scene_name = dss.staticDataModel.getNameForScene(0, 0, scene);
    var gridTextInactive = Ext.String.format2(_("only when {name} is inactive"), {'name': scene_name});
    return gridTextInactive;
}

Ext.define('DSS.addon.HLEEdit', {

    extend : 'DSS.addon.Framework',

    config : {
        appName : 'User defined action',
        appId : 'hleedit',
        appVersion : '1.4.4.0',
        appIcon : 'default_icon.png'
    },

    getHelp : function() {
        return Ext.create('Ext.Component', {
            whiteSpace : 'normal',
            padding : '0 20 0 0', //top right bottom left
            loader : {
                url: 'locale/' + dss.staticDataModel.activeLanguage + '/help.html',
                autoLoad : true,
                disableCaching : false
            }
        });
    },
    megrid:null,
    getContent : function() {
        var me=this;

        var conditions = [{type:'singleItem',title:_('state_daynight'),entry:[
                              {type:'state',name:'daylight',value:true,title:Ext.String.format2(_('daynight_bright'),{
                                        duskTime:dss.staticDataModel.getDusk(),
                                        dawnTime:dss.staticDataModel.getDawn()
                                    }),gridText:Ext.String.format2(_('grid_daynight_twilight_true'),{
                                        duskTime:dss.staticDataModel.getDusk(),
                                        dawnTime:dss.staticDataModel.getDawn()
                                    })},
                              {type:'state',name:'daylight',value:false,title:Ext.String.format2(_('daynight_dark'),{
                                    duskTime:dss.staticDataModel.getDusk(),
                                    dawnTime:dss.staticDataModel.getDawn()
                                }),gridText:Ext.String.format2(_('grid_daynight_twilight_false'),{
                                    duskTime:dss.staticDataModel.getDusk(),
                                    dawnTime:dss.staticDataModel.getDawn()
                                })},
                              {type:'state',name:'daynight',value:true,title:Ext.String.format2(_('daynight_day'),{
                                    sunriseTime:dss.staticDataModel.getSunrise(),
                                    sunsetTime:dss.staticDataModel.getSunset()
                                }),gridText:Ext.String.format2(_('grid_daynight_daynight_true'),{
                                    sunriseTime:dss.staticDataModel.getSunrise(),
                                    sunsetTime:dss.staticDataModel.getSunset()
                                })},
                              {type:'state',name:'daynight',value:false,title:Ext.String.format2(_('daynight_night'),{
                                    sunriseTime:dss.staticDataModel.getSunrise(),
                                    sunsetTime:dss.staticDataModel.getSunset()
                                }),gridText:Ext.String.format2(_('grid_daynight_daynight_false'),{
                                    sunriseTime:dss.staticDataModel.getSunrise(),
                                    sunsetTime:dss.staticDataModel.getSunset()
                                })}]},
                          {type:'timeframe',title:_('Timeframe')},
                          {type:'weekdays',title:_('Weekdays')},
                          {type:'dateframe',title:_('Dateframe')},
                          {type:'vdcDeviceState',title:_('device states')},                          
                          {type:'userDefinedState',title:_('Custom states')},
                          {type:'deviceState',title:_('automatic devices')},
                          {type:'singleItem',title:_('state_presence'),entry: [
                              {type:'state',name:'presence',value:'present',title:_('presence_present'),gridText:_('grid_presence_present')},
                              {type:'state',name:'presence',value:'absent',title:_('presence_absent'),gridText:_('grid_presence_absent')}]},
                          {type:'singleItem',title:_('state_holiday'),entry: [
                              {type:'state',name:'holiday',value:'on',title:_('holiday_on'),gridText:_('grid_holiday_on')},
                              {type:'state',name:'holiday',value:'off',title:_('holiday_off'),gridText:_('grid_holiday_off')}]},
                          {type:'multiItem',title:_('weather trigger'),entry: [
                              {type:'state',name:'rain',value:'1',title:_('rain active'),gridText:_('grid_rain_active'),group:1},
                              {type:'state',name:'rain',value:'2',title:_('rain inactive'),gridText:_('grid_rain_inactive'),group:1},
                              {type:'state',name:'wind',value:'1',title:_('wind active'),gridText:_('grid_wind_active'),group:2},
                              {type:'state',name:'wind',value:'2',title:_('wind inactive'),gridText:_('grid_wind_inactive'),group:2},
                              {type:'state',name:'hail',value:'1',title:_('hail active'),gridText:_('grid_hail_active'),group:3},
                              {type:'state',name:'hail',value:'2',title:_('hail inactive'),gridText:_('grid_hail_inactive'),group:3},
                              {type:'state',name:'frost',value:'1',title:_('frost active'),gridText:_('grid_frost_active'),group:4},
                              {type:'state',name:'frost',value:'2',title:_('frost inactive'),gridText:_('grid_frost_inactive'),group:4},
                              {type:'state',name:'sun',value:'1',title:_('sun active'),gridText:_('grid_sun_active'),group:5},
                              {type:'state',name:'sun',value:'2',title:_('sun inactive'),gridText:_('grid_sun_inactive'),group:5}]},
                              {type:'state',name:'service',value:'1',title:_('service active'),gridText:_('grid_service_active'),group:6},
                              {type:'state',name:'service',value:'2',title:_('service inactive'),gridText:_('grid_service_inactive'),group:6},
                         {type:'multiItem',title:_('Security'),entry: [
                              {type:'state',name:'fire',value:'1',title:_('fire active'),gridText:_('grid_fire_active'),group:1},
                              {type:'state',name:'fire',value:'2',title:_('fire inactive'),gridText:_('grid_fire_inactive'),group:1},
                              {type:'state',name:'panic',value:'1',title:_('panic_active'),gridText:_('grid_panic_active'),group:2},
                              {type:'state',name:'panic',value:'2',title:_('panic_inactive'),gridText:_('grid_panic_inactive'),group:2},
                              {type:'state',name:'alarm',value:'1',title:_('alarm_active'),gridText:_('grid_alarm_active'),group:3},
                              {type:'state',name:'alarm',value:'1',title:getStateGridTextActive(74),gridText:getStateGridTextActive(74), group:3},
                              {type:'state',name:'alarm',value:'2',title:getStateGridTextInactive(74),gridText:getStateGridTextInactive(74),group:3},
                              {type:'state',name:'alarm2',value:'1',title:getStateGridTextActive(83),gridText:getStateGridTextActive(83),group:4},
                              {type:'state',name:'alarm2',value:'2',title:getStateGridTextInactive(83),gridText:getStateGridTextInactive(83),group:4},
                              {type:'state',name:'alarm3',value:'1',title:getStateGridTextActive(84),gridText:getStateGridTextActive (84),group:5},
                              {type:'state',name:'alarm3',value:'2',title:getStateGridTextInactive(84),gridText:getStateGridTextInactive(84),group:5},
                              {type:'state',name:'alarm4',value:'1',title:getStateGridTextActive(85),gridText:getStateGridTextActive(85),group:6},
                              {type:'state',name:'alarm4',value:'2',title:getStateGridTextInactive(85),gridText:getStateGridTextInactive(85),group:6}]},
                          {type:'zonestate',title:_('ZoneState')},
                          {type:'clusterstate',title:_('Cluster State')}];

        var climateSettings = dss.staticDataModel.getClimateSettings();
        if (climateSettings != null && climateSettings.capabilities != null && climateSettings.capabilities.coolingSupported == true) {
            conditions.push({type:'multiItem',title:_('Climate system'),entry: [
                                {type:'state',name:'heating_water_system',value:'0',title:_('off'),gridText:_('grid_climate_off'),group:1},
                                {type:'state',name:'heating_water_system',value:'1',title:_('heating on'),gridText:_('grid_climate_heating'),group:1},
                                {type:'state',name:'heating_water_system',value:'2',title:_('cooling on'),gridText:_('grid_climate_cooling'),group:1}]
            });
        } else {
            conditions.push({type:'multiItem',title:_('Climate system'),entry: [
                                {type:'state',name:'heating_water_system',value:'0',title:_('off'),gridText:_('grid_climate_off'),group:1},
                                {type:'state',name:'heating_water_system',value:'1',title:_('heating on'),gridText:_('grid_climate_heating'),group:1}]
            });
        }

        var oConfig={
            defaultSortCol:'name',
            disabledProperty:'disabled',            
            storeConfig:{
                storeId : 'hleStore',
                fields : [ {name:'id'}, 
                           {name:'disabled', sortType:'asBoolean'}, 
                           {name:'name', sortType:'asUCText'}, 
                           {name:'actions', sortType:'asAction'}, 
                           {name:'lastSaved', sortType:'asTimestamp'}, 
                           {name:'lastExecuted', sortType:'asTimestamp'},
                           {name: 'conditions',  sortType: 'asCondition' },
                           { name: 'grouping',  sortType: 'asUCText' },
                           {name:'source', sortType:'asUCText'}]
            },
            multiselect:true,
            columns:[
                {
                    header : _('Active'),
                    dataIndex : 'disabled',
                    resizable : false,
                    xtype: 'disabledcolumn', 
                    width : 50
                },{
                    header : _('Name'),
                    dataIndex : 'name',
                    xtype: 'textcolumn',
                    width : 200
                },{
                    header : _('Activity'),
                    dataIndex : 'actions', 
                    maintainFlex : true,
                    flex : 5,
                    xtype: 'actioncolumn'
                },{
                    header: _('Conditions'),
                    dataIndex: 'conditions', 
                    xtype: 'conditioncolumn',
                    maintainFlex: true,
                    flex: 3,
                    customRendererOptions:{hideEnabled:true,hideWeekdays:false,hideGenericSystemStates:false,hideZoneStates:false,hideTimeframe:false,filterSystemState:[],
                        stateTitles:[{name:'daylight',value:true,title:_('grid_daylight_true')},{name:'daylight',value:false,title:_('grid_daylight_false')},
                                     {name:'daynight',value:true,title:_('grid_daynight_true')},{name:'daynight',value:false,title:_('grid_daynight_false')},
                                     {name:'panic',value:'1',title:_('grid_panic_active')},{name:'panic',value:'2',title:_('grid_panic_inactive')},
                                     {name:'presence',value:'present',title:_('grid_presence_present')},{name:'presence',value:'absent',title:_('grid_presence_absent')},
                                     {name:'fire',value:'1',title:_('grid_fire_active')},{name:'fire',value:'2',title:_('grid_fire_inactive')},
                                     {name:'hail',value:'1',title:_('grid_hail_active')},{name:'hail',value:'2',title:_('grid_hail_inactive')},
                                     {name:'wind',value:'1',title:_('grid_wind_active')},{name:'wind',value:'2',title:_('grid_wind_inactive')},
                                     {name:'rain',value:'1',title:_('grid_rain_active')},{name:'rain',value:'2',title:_('grid_rain_inactive')},
                                     {name:'frost',value:'1',title:_('grid_frost_active')},{name:'frost',value:'2',title:_('grid_frost_inactive')},
                                     {name:'sun',value:'1',title:_('grid_sun_active')},{name:'sun',value:'2',title:_('grid_sun_inactive')},
                                     {name:'service',value:'1',title:_('grid_service_active')},{name:'service',value:'2',title:_('grid_service_inactive')},
                                     {name:'alarm',value:'1',title:getStateGridTextActive(74)},{name:'alarm', value:'2',title:getStateGridTextInactive(74)},
                                     {name:'alarm2',value:'1',title:getStateGridTextActive(83)},{name:'alarm2',value:'2',title:getStateGridTextInactive(83)},
                                     {name:'alarm3',value:'1',title:getStateGridTextActive(84)},{name:'alarm3',value:'2',title:getStateGridTextInactive(84)},
                                     {name:'alarm4',value:'1',title:getStateGridTextActive(85)},{name:'alarm4',value:'2',title:getStateGridTextInactive(85)},
                                     {name:'holiday', value:'on',title:_('grid_holiday_on')},{name:'holiday', value:'off',title:_('grid_holiday_off')},
                                     {name:'heating_water_system', value:'0',title:_('grid_climate_off')},{name:'heating_water_system', value:'1',title:_('grid_climate_heating')},{name:'heating_water_system', value:'2',title:_('grid_climate_cooling')}]}
                },{
                    header : _('last edit'),
                    dataIndex : 'lastSaved',
                    xtype: 'timestampcolumn',
                    width : 200
                }/*,
                {
                    header : _('last executed'),
                    dataIndex : 'lastExecuted',
                    xtype:'timestampcolumn',
                    width : 200
                }*/
            ],
            buttonText: {
                newButton: _('New action'),
                editButton: _('Edit action'),
                enableButton:_('Activate action'),
                disableButton:_('Deactivate action'),
                testButton: _('Test action'),
                deleteButton: _('Delete action')
            },
            editorConfig:{
                title : _('Edit user defined action'),
                staticControls : [ {
                    type : 'text',
                    label : _('Name'),
                    id : 'name'
                }, {
                    type : 'hidden',
                    label : 'id',
                    id : 'id'
                },{
                    type : 'hidden',
                    label : 'source',
                    id : 'source'
                }],
                tabControls : [ {
                    tabLabel : _('Activity'),
                    tabDescription : _('Choose the activities, which should be executed.'),
                    content : [ {
                        type : 'ActionConfigurator',
                        id : 'actions',
                    	heatingOperationMode:true,
                        flex : true,
                        noHighlevel:true,
                        withDelay : true
                    } ]
                }, {
                    tabLabel : _('Conditions'),
                    content : [{
                        type : 'ConditionConfigurator',
                        id : 'conditions',
                        tabDescription : _('Choose the conditions.'),
                            conditions:conditions,
                            flex : true
                    }]
                } ]
            },
            listeners:{
                'loadDataStore':function (grid){
                    me.loadParameter(grid);
                },
                'saveEntry':function (grid,values){                        
                    me.waitAndReload(grid,{
                        actions : 'save',
                        value : Ext.JSON.encode(values)
                    });
                },
                'enableEntry':function (grid,entry){
                	var idVal;
                	if (entry.get!=null) {
                		idVal=entry.get('id');
                	} else {
                		idVal=[];
                		for (var iIndex=0;iIndex<entry.length;iIndex++) 
                			if (entry[iIndex]!=null)
                				idVal.push(entry[iIndex].get('id'));
                	}
                    me.waitAndReload(grid,{
                        actions : 'enable',
                        id : Ext.JSON.encode(idVal)
                    });
                 },
                'disableEntry':function (grid,entry){
                	var idVal;
                	if (entry.get!=null) {
                		idVal=entry.get('id');
                	} else {
                		idVal=[];
                		for (var iIndex=0;iIndex<entry.length;iIndex++) 
                			if (entry[iIndex]!=null)
                				idVal.push(entry[iIndex].get('id'));
                	}
                    me.waitAndReload(grid,{
                        actions : 'disable',
                        id : Ext.JSON.encode(idVal)
                    });
                },
                'deleteEntry':function (grid,entry){
                	var idVal;
                	if (entry.get!=null) {
                		idVal=entry.get('id');
                	} else {
                		idVal=[];
                		for (var iIndex=0;iIndex<entry.length;iIndex++) 
                			if (entry[iIndex]!=null)
                				idVal.push(entry[iIndex].get('id'));
                	}
                    me.waitAndReload(grid,{
                        actions : 'delete',
                        id : Ext.JSON.encode(idVal)
                    });
                },
                'testEntry':function (grid,entry){
                    dss.raiseEvent('highlevelevent', {
                        id : entry.get('id')
                    });
                    window.setTimeout(function() {
                        me.loadParameter(grid);
                    }, 2 * 1000);
                },
                'selectionChanged': function(grid, selections) {
                	var fSource=(selections.length>0);
                	for (var iIndex=0;iIndex<selections.length;iIndex++) {
                		if (selections[iIndex].get('source')!=null)
                			if (selections[iIndex].get('source')!='')
                				if (selections[iIndex].get('source')!='system-addon-user-defined-actions')
                					 if (getParameterByName('mode')!='expert')
                						 fSource=false;
                	}
                    grid.btnEdit.setDisabled(!fSource);                            
                    grid.btnDelete.setDisabled(!fSource);                      
                }

            }
        };
        
        if (getParameterByName('mode')=='expert') {
            oConfig.editorConfig.tabControls[0].content[0].enableUsergroup=true;
            oConfig.editorConfig.tabControls[0].content[0].withDelay=true;
            oConfig.editorConfig.tabControls[0].content[0].allowNoAction=true;
            oConfig.editorConfig.tabControls[0].content[0].noHighlevel=false;
            oConfig.editorConfig.tabControls[0].content[0].heatingOperationMode=true;
            oConfig.editorConfig.tabControls[0].content[0].UMRExtendetFunctions=true;
            oConfig.editorConfig.tabControls[0].content[0].heatingExtendedOperationMode=true;
            oConfig.editorConfig.tabControls[1].content[0].ignorePresence=true;
            oConfig.editorConfig.tabControls[1].content[0].fFeatureEnabled={
                    enabler:true,
                    weekdays:true,
                    timeframe:true,
                    systemstate:true,
                    zonestate:true,
                    userDefinedStates:true};

            oConfig.columns.push({
                header : _('source'),
                dataIndex : 'source',
                xtype:'textcolumn',
                width : 200
            });
            oConfig.grouperField='grouping';
        } 
        return Ext.create('DSS.gridPanel', oConfig);
    },
    waitAndReload : function(grid,parameter) {
    	grid.setStatus(_('Saving ...'));
    	
    	var me = this;
    	dss.pushAndQuery('system-addon-user-defined-actions-config',parameter,
    			         'system-addon-user-defined-actions-saved',function(eventData) {
    		 me.loadParameter(grid);
    		 return true;
    	});
  
    },   
    loadParameter : function(grid) {
    	grid.setStatus(_('Loading ...'));
        dss.staticDataModel.initHLECache();
        var sString = dss.ajaxSyncRequest('/json/property/query', {
            'query' : '/usr/events/*(*)/*(*)/*(*)/*(*)/*(*)'
        });
        var result = Ext.JSON.decode(sString);
        
        var oResultObject=dss.parsePropertyQueryAnswer(result,[{name:'name',type:'simple'},
                                                               {name:'source',type:'simple'},
                                                               {name:'id',type:'simple'},
                                                               {name:'disabled',type:'condition/enabled'},
                                                               {name:'lastSaved',type:'simple'},
                                                               {name:'lastExecuted',type:'simple'},
                                                               {name: 'conditions', type: 'condition'},
                                                               {name:'actions',type:'action'}]);
        
        for (var iIndex=0;iIndex<oResultObject.length;iIndex++) {
            oResultObject[iIndex].grouping=unescape(oResultObject[iIndex].name).split(' ')[0];
        }
        
        if (oResultObject!=null) {
            grid.loadParameter(oResultObject);
        }
        grid.setStatus(_('Ready'));
    }
});

Ext.onReady(function() {
    dss.buildUpLang(['locale/{languageSuffix}/user-defined-actions.po']);
    var HLEEdit = Ext.create('DSS.addon.HLEEdit');
    HLEEdit.initPage();
});
